from odin_stream_cpp import FixedSizeParameter,ParameterSet


parameters = [
    FixedSizeParameter(index=10005, data=b'myta'),
    FixedSizeParameter(index=20005, data=b'myata'),
    FixedSizeParameter(index=30005, data=b'my_data'),
    FixedSizeParameter(index=40005, data=b'my_dta'),
    FixedSizeParameter(index=50005, data=b'my_data'),
]

set = ParameterSet(10)
set.add_list(parameters)

identifier = set.generate_identifier_packet()
data = set.generate_data_packet(10)

# Reconstruct the set from the identifier packet
idw = ParameterSet.parse_identifier_packet(identifier)
# Get list of ids
print(f"Identifier: {idw.indices}")
# Get list of data packets
print(f"Identifier: {idw.parse_data_packet(data)}")





import odin_stream_cpp 
from odin_stream_cpp import PrimitiveTypeDescriptor, PrimitiveType, StructDescriptor,TypeDescriptors
import polars as pl

with open("test/dataset.hex", "r") as f:
    data = f.read()
    data = data.split("\n")
    data = [bytes.fromhex(i) for i in data]



vec7_descriptor = StructDescriptor()
vec7_descriptor.add_member("timestamp", PrimitiveTypeDescriptor(odin_stream_cpp.UINT32))
vec7_descriptor.add_member("sequence_number", PrimitiveTypeDescriptor(odin_stream_cpp.UINT16))
vec7_descriptor.add_member("accel_x", PrimitiveTypeDescriptor(odin_stream_cpp.FLOAT32))
vec7_descriptor.add_member("accel_y", PrimitiveTypeDescriptor(odin_stream_cpp.FLOAT32))
vec7_descriptor.add_member("accel_z", PrimitiveTypeDescriptor(odin_stream_cpp.FLOAT32))
vec7_descriptor.add_member("gyro_x", PrimitiveTypeDescriptor(odin_stream_cpp.FLOAT32))
vec7_descriptor.add_member("gyro_y", PrimitiveTypeDescriptor(odin_stream_cpp.FLOAT32))
vec7_descriptor.add_member("gyro_z", PrimitiveTypeDescriptor(odin_stream_cpp.FLOAT32))
vec7_descriptor.add_member("temperature", PrimitiveTypeDescriptor(odin_stream_cpp.FLOAT32))

print(type(vec7_descriptor))
types = TypeDescriptors({0xE0C00000: vec7_descriptor,0xE0B10000: vec7_descriptor, 0xE0B20000: vec7_descriptor, 0xE0B30000: vec7_descriptor, 0xE0B40000: vec7_descriptor})


processor = odin_stream_cpp.StreamProcessor(types)
import time
start = time.time()
processor.process_bytes_list(data)



sert = (processor.get_parameter_set(6684))
# print(sert.flush_to_arrow_table())
# data = pl.from_arrow(sert.flush_to_arrow_table())
# to csv
# data.write_csv("test/dataset.csv")
print(pl.from_arrow(sert.flush_to_arrow_table()))
end = time.time()
print(f"Execution time: {end - start} s")


import pyarrow
import odin_stream_cpp
import polars as pl
import time

with open("test/dataset.hex", "r") as f:
    data = f.read()
    # split newline
    data = data.split("\n")
    # Decode hex
    data = [bytes.fromhex(i) for i in data]


start = time.time()
    # result = pl.from_arrow(odin_stream.test_create())
processor = odin_stream_cpp.StreamProcessor()
processor.process_bytes_list(data)

data1 = pl.from_arrow(processor.get_parameter_set(25762).flush_to_arrow_table())
data2 = pl.from_arrow(processor.get_parameter_set(6684).flush_to_arrow_table())

# Save to compressed parquet
data1.write_parquet("test/dataset1.parquet", compression="zstd",compression_level=22)
data2.write_parquet("test/dataset2.parquet", compression="zstd",compression_level=22)
end = time.time()
print(f"Execution time: {end - start} ms")
# print(result)

print(data1)





import polars as pl
from odin_db import OdinDBModel

import odin_stream_cpp

from .odin_db_mapping import (
    db_type_to_steam_type,
    get_type_dict,
    odin_db_to_flat_list,
    parameter_to_odin_param,
)


class StreamProcessor:
    processor: odin_stream_cpp.StreamProcessor

    def __init__(self, odin_db: OdinDBModel, silent_errors: bool = False):
        self.type_dict = get_type_dict(odin_db)

        self.parameter_map = odin_stream_cpp.ParameterMapDescriptor()

        for parameter in odin_db_to_flat_list(odin_db.root):
            param = parameter_to_odin_param(parameter, self.type_dict)
            self.parameter_map.add_parameter(param)

        self.processor = odin_stream_cpp.StreamProcessor(
            self.parameter_map, silent_errors
        )

    def process_bytes_list(self, data: list[bytes]):
        """
        Process a list of bytes and add them to the processor.
        """
        self.processor.process_packets(data)

    def flush(self) -> dict[int, pl.DataFrame]:
        sets = {}
        for key in self.processor.get_parameter_set_identifiers():
            param_set = self.processor.get_parameter_set(key)

            # print(f"Processing parameter set {key} with {param_set} parameters")

            arrow_table = param_set.flush_to_arrow_table()
            
            # print(f"Arrow table for parameter set {arrow_table} rows")

            sets[key] = pl.from_arrow(
                arrow_table
            )
        return sets

    @property
    def statistics(self) -> odin_stream_cpp.OdinStreamStatistics:
        return self.processor.statistics



import odin_db
from odin_db import OdinDBModel, ODINDBModelType, OdinDBTypeDefinitionModel

import odin_stream_cpp


def odin_db_to_flat_list(
    group: odin_db.OdinDBParameterGroupModel,
) -> list[odin_db.OdinDBParameterModel]:
    """
    Convert OdinDBModel to a flat list of ParameterSets.
    """
    flat_list = []
    for parameter in group.parameters:
        if isinstance(parameter, odin_db.OdinDBParameterModel):
            flat_list.append(parameter)
        elif isinstance(parameter, odin_db.OdinDBParameterGroupModel):
            flat_list.extend(odin_db_to_flat_list(parameter))
    return flat_list


# A composite type is can contain one or more primitive subtypes, in a later stage they can contain other composite types
# def odin_type_dedintion_to_non_composite(
#     structure: "dict[str, OdinDBTypeDefinitionModel | ODINDBModelType]",
# ) -> "dict[str, ODINDBModelType]":
#     """
#     Convert OdinDBModel to a flat list of TypeDefinitions.
#     """
#     flat_structure = {}

#     for name, sub_type in structure.items():
#         print(name)
#         if isinstance(sub_type.structure, dict):
#             print(f"Converting member {name} of type {sub_type.name}")
#         # print(f"Converting member {name} of type {sub_type.name}")

#     return master_type


def db_type_to_steam_type(
    name: str, type: odin_db.OdinDBTypeDefinitionModel
) -> odin_stream_cpp.CompositeTypeDescriptor | odin_stream_cpp.PrimitiveTypeDescriptor:
    composite_descriptor = odin_stream_cpp.CompositeTypeDescriptor()

    if isinstance(type.structure, odin_db.ODINDBModelType):
        return odin_stream_cpp.PrimitiveTypeDescriptor.get_by_name(
            type.structure.name.lower()
        )

    elif isinstance(type.structure, dict):
        for sub_name, sub_type in type.structure.items():
            if isinstance(sub_type, odin_db.ODINDBModelType):
                descriptor = odin_stream_cpp.PrimitiveTypeDescriptor.get_by_name(
                    sub_type.name.lower()
                )
                composite_descriptor.add_member(sub_name, descriptor)

            elif isinstance(sub_type.structure, odin_db.ODINDBModelType):
                descriptor = odin_stream_cpp.PrimitiveTypeDescriptor.get_by_name(
                    sub_type.structure.name.lower()
                )
                composite_descriptor.add_member(sub_name, descriptor)

            elif isinstance(sub_type.structure, dict):
                for sub_sub_name, sub_sub_type in sub_type.structure.items():
                    if isinstance(sub_sub_type, odin_db.ODINDBModelType):
                        descriptor = (
                            odin_stream_cpp.PrimitiveTypeDescriptor.get_by_name(
                                sub_sub_type.name.lower()
                            )
                        )
                        composite_descriptor.add_member(
                            f"{sub_name}_{sub_sub_name}", descriptor
                        )

                    elif isinstance(sub_sub_type.structure, odin_db.ODINDBModelType):
                        descriptor = (
                            odin_stream_cpp.PrimitiveTypeDescriptor.get_by_name(
                                sub_sub_type.structure.name.lower()
                            )
                        )
                        composite_descriptor.add_member(
                            f"{sub_name}_{sub_sub_name}", descriptor
                        )

                    elif isinstance(sub_sub_type.structure, dict):
                        raise ValueError(
                            "maximum depth of composite types is 2, please flatten the structure"
                        )

    return composite_descriptor


def get_type_dict(
    db: odin_db.OdinDBModel,
) -> dict[
    str,
    odin_stream_cpp.CompositeTypeDescriptor | odin_stream_cpp.PrimitiveTypeDescriptor,
]:
    assert db.types is not None, "OdinDBModel types should not be None"

    type_dict = {}
    for type_name, type in db.types.items():
        type_dict[type_name] = db_type_to_steam_type(type_name, type)

    for type_name, _type in odin_db.ODINDBModelType.__members__.items():
        type_dict[type_name.lower()] = (
            odin_stream_cpp.PrimitiveTypeDescriptor.get_by_name(_type.name.lower())
        )

    return type_dict


def parameter_to_odin_param(
    parameter: odin_db.OdinDBParameterModel,
    type_dict: dict[
        str,
        odin_stream_cpp.CompositeTypeDescriptor
        | odin_stream_cpp.PrimitiveTypeDescriptor,
    ],
) -> odin_stream_cpp.ParameterDescriptor:
    element_type = type_dict.get(parameter.element_type)

    if element_type is None:
        return odin_stream_cpp.ParameterDescriptor.unknown_type(
            parameter.global_id, parameter.global_name
        )

    return odin_stream_cpp.ParameterDescriptor(
        parameter.global_id, parameter.global_name, element_type
    )


#         """
#         Convert OdinDBModel to TypeDescriptors for StreamProcessor.
#         """
#         descriptors : dict[str, odin_stream_cpp.StructDescriptor|odin_stream_cpp.PrimitiveTypeDescriptor] = {}
#         assert odin_db.types is not None, "OdinDBModel types should not be None"

#         for type_name, _type in odin_db.types.items():
#             structure = odin_stream_cpp.StructDescriptor()

#             for name, odin_type in _type.structure.items():
#                 if isinstance(odin_type, ODINDBModelType):
#                     structure.add_member(name, StreamProcessor.odin_db_type_to_odin_stream_type(odin_type))

#                 elif isinstance(odin_type, OdinDBTypeDefinitionModel):
#                     # structure.add_member(name, odin_stream_cpp.StructDescriptor())
#                     print(f"SKIPPING member {name} of type {odin_type}")

#             print(f"Registerd type {type_name} with size {structure.get_size()}")

#             descriptors[type_name] = structure
#             #

#         # Add primitive types
#         for type_name, _type in ODINDBModelType.__members__.items():
#             descriptors[type_name.lower()] = StreamProcessor.odin_db_type_to_odin_stream_type(_type)

#         parameter_descriptors = {}
#         for parameter in StreamProcessor.odin_db_to_flat_list(odin_db.root):
#             element_type = descriptors.get(parameter.element_type)

#             if element_type is None:
#                 print(f"Type {parameter.element_type} not found in descriptors, skipping parameter {parameter.name}")
#                 continue

#             parameter_descriptors[parameter.global_id] = element_type

#         # Print all parameter descriptors
#         for parameter_id, descriptor in parameter_descriptors.items():
#             print(f"Parameter ID: {parameter_id:08X} Size: {descriptor.get_size()}")

#         return odin_stream_cpp.TypeDescriptors(parameter_descriptors)



import time

import odin_db

from odin_stream import StreamProcessor

ODIN_PATH = r"C:\Users\fvernieuwe\Downloads\OD.odin"
with open(ODIN_PATH, "rb") as f:
    odin_db_data = odin_db.OdinDBModel.model_validate_json(f.read())

processor = StreamProcessor(odin_db_data, silent_errors=True)

with open("test/dataset.hex", "r") as f:
    data = f.read()
    data = data.split("\n")
    data = [bytes.fromhex(i) for i in data]


start = time.time()
processor.process_bytes_list(data[100:10000])

end = time.time()
result = processor.flush()
print(f"Execution time: {end - start} s")

print(processor.statistics)
print("FINISHED")


import pyarrow
import odin_stream_cpp
import polars as pl
import time

paraa = odin_stream_cpp.FixedSizeParameter(index=10005, size=4)

paraa.add_data(b"1234")
paraa.add_data(b"1234")
paraa.add_data(b"1234")
paraa.add_data(b"1234")
paraa.add_data(b"1234")
paraa.add_data(b"1234")
paraa.add_data(b"1234")
paraa.add_data(b"1234")
paraa.add_data(b"1234")

# Dump
print(paraa)

# with open("test/dataset.hex", "r") as f:
#     data = f.read()
#     # split newline
#     data = data.split("\n")
#     # Decode hex
#     data = [bytes.fromhex(i) for i in data]


# start = time.time()
#     # result = pl.from_arrow(odin_stream.test_create())
# processor = odin_stream.StreamProcessor()
# processor.process_bytes_list(data)

# end = time.time()
# print(f"Execution time: {end - start} ms")
# # print(result)


